@@ -34,6 +34,7 @@
 int C_A_D = 1;
 
 extern void adjust_clock(void);
+extern void DAC960_Finalize(void);
 extern void gdth_halt(void);
 
 asmlinkage int sys_ni_syscall(void)
@@ -171,7 +172,7 @@ asmlinkage int sys_prof(void)
 #endif
 
 extern void hard_reset_now(void);
-extern asmlinkage sys_kill(int, int);
+extern asmlinkage int sys_kill(int, int);
 
 /*
  * Reboot system call: for obvious reasons only root may call it,
@@ -188,6 +189,9 @@ asmlinkage int sys_reboot(int magic, int magic_too, int flag)
 	if (magic != 0xfee1dead || magic_too != 672274793)
 		return -EINVAL;
 	if (flag == 0x01234567) {
+#ifdef CONFIG_BLK_DEV_DAC960
+		DAC960_Finalize();
+#endif
 #ifdef CONFIG_SCSI_GDTH
 		gdth_halt();
 #endif
@@ -197,10 +201,16 @@ asmlinkage int sys_reboot(int magic, int magic_too, int flag)
 	else if (!flag)
 		C_A_D = 0;
 	else if (flag == 0xCDEF0123) {
+#ifdef CONFIG_BLK_DEV_DAC960
+		DAC960_Finalize();
+#endif
+#ifdef CONFIG_SCSI_GDTH
+		gdth_halt();
+#endif
 		printk(KERN_EMERG "System halted\n");
 		sys_kill(-1, SIGKILL);
 #if defined(CONFIG_APM) && defined(CONFIG_APM_POWER_OFF)
-		apm_set_power_state(APM_STATE_OFF);
+		apm_power_off();
 #endif
 		do_exit(0);
 	} else
@@ -216,6 +226,9 @@ asmlinkage int sys_reboot(int magic, int magic_too, int flag)
 void ctrl_alt_del(void)
 {
 	if (C_A_D) {
+#ifdef CONFIG_BLK_DEV_DAC960
+		DAC960_Finalize();
+#endif
 #ifdef CONFIG_SCSI_GDTH
 		gdth_halt();
 #endif
@@ -663,7 +676,8 @@ asmlinkage int sys_getgroups(int gidsetsize, gid_t *grouplist)
 	int i;
 	int * groups;
 
-	if (gidsetsize < 0)
+	/* Avoid an integer overflow on systems with 32 bit gid_t (Alpha) */
+	if (gidsetsize & ~0x3FFFFFFF)
 		return -EINVAL;
 	groups = current->groups;
 	for (i = 0 ; i < NGROUPS ; i++) {
